var BOSH_SERVICE = 'http://localhost:5280/http-bind/';
var show_log = true;

Strophe.addNamespace("ADHOC", "http://jabber.org/protocol/commands");

var localJID = null;
var connection   = null;
var adhoc_status = {
    sessionid: null,
    cmdNode: null,
    queryJID: null
};

function log(msg) {
    var entry = $('<div></div>').append(document.createTextNode(msg));
    $('#log').append(entry);
}

function rawInput(data) {
    log('RECV: ' + data);
}

function rawOutput(data) {
    log('SENT: ' + data);
}

function onConnect(status) {
    if (status == Strophe.Status.CONNECTING) {
        log('Strophe is connecting.');
    } else if (status == Strophe.Status.CONNFAIL) {
        log('Strophe failed to connect.');
        showConnect();
    } else if (status == Strophe.Status.DISCONNECTING) {
        log('Strophe is disconnecting.');
    } else if (status == Strophe.Status.DISCONNECTED) {
        log('Strophe is disconnected.');
        showConnect();
    } else if (status == Strophe.Status.AUTHFAIL) {
        log('Authentication failed');
        if (connection) {
            connection.disconnect();
        }
    } else if (status == Strophe.Status.CONNECTED) {
        log('Strophe is connected.');
        adhoc_status.queryJID = connection.domain;
        $('#queryJID').val(adhoc_status.queryJID);
        $('#query').show();
        checkFeatures("#output");
    }
}

function addNote(elem, text, type) {
    if (!type) {
       type = "info";
    }
    text = text.replace(/\n/g, "<br/>");
    $(elem).append("<p class='" + type + "Note'>" + text + "</p>");
}

function addForm(elem, x) {
    var form = $("<form action='#'/>");
    form.submit(function(event) {
        executeCommand("execute", serializeToDataform('form'), function(e) { displayResult(elem, e) });
        event.preventDefault();
    });
    var fieldset = $("<fieldset/>");
    form.append(fieldset);
    if ($(x).find("title").length > 0)
        $("<legend/>").text($(x).find("title").text()).appendTo(fieldset);
    if ($(x).find("instructions").length > 0)
        $("<p/>").text($(x).find("instructions").text()).appendTo(fieldset);
    $(x).find("field").each(function() {
        var item = null;
        var type = $(this).attr("type");
        if($(this).attr("label")) {
            $("<label/>").text($(this).attr("label")).attr("for", $(this).attr("var")).appendTo(fieldset);
            $("<br/>").appendTo(fieldset);
        }
        switch(type) {
        case "hidden":
            item = $("<input type='hidden'/>");
            break;
        case "boolean":
            item = $("<input type='checkbox'/>");
            break;
        case "text-multi":
            item = $("<textarea rows='10' cols='70'/>");
            break;
        case "text-single":
            item = $("<input type='text'/>");
            break;
        case "fixed":
            item = $("<input type='text'/>").attr("readonly",true);
            break;
        case "jid-multi":
            item = $("<textarea rows='10' cols='70'/>");
            break;
        case "jid-single":
            item = $("<input type='text'/>");
            break;
        case "list-multi":
            item = $("<select multiple='multiple'/>");
            $(this).find("option").each(function() {
                $("<option/>").val($(this).find("value").text()).text($(this).attr("label")).appendTo(item);
            });
            break;
        case "list-single":
            item = $("<select/>");
            $(this).find("option").each(function() {
                $("<option/>").val($(this).find("value").text()).text($(this).attr("label")).appendTo(item);
            });
            break;
        case "text-private":
            item = $("<input type='password'/>");
            break;
        default:
            item = $("<input/>");
        }
        item.addClass("df-item");
        if ($(this).find("value").length > 0) {
            var value = null;
            if ((type == "text-multi") || (type == "jid-multi")) {
                value = "";
                $(this).find("value").each(function() {
                    value = value + $(this).text() + "\n";
                });
                item.val(value);
            } else if (type == "list-multi") {
                $(this).children("value").each(function() {
                    item.children('option[value="' + $(this).text() + '"]').each(function() {
                        $(this).attr("selected", "selected");
		    });
                });
            } else {
                item.val($(this).find("value").text());
            }
        }
        if ($(x).attr("type") == "result")
            item.attr("readonly", true);
        if ($(this).attr("var")) {
            item.attr("name", $(this).attr("var"));
            item.attr("id", $(this).attr("var"));
        }
        fieldset.append(item);
        if (type != "hidden")
            fieldset.append("<br/>");
    });
    $(elem).append(form);
}

function serializeToDataform(form) {
    st = $build("x", {"xmlns": "jabber:x:data", "type": "submit"});
    $(form).find(".df-item").each(function(){
        st.c("field", {"var": $(this).attr("name")});
        if (this.nodeName.toLowerCase() == "select" && this.multiple) {
            for (var i = 0; i < this.options.length; i++)
                if (this.options[i].selected)
                    st.c("value").t(this.options[i].text).up();
        } else if (this.nodeName.toLowerCase() == "textarea") {
            var sp_value = this.value.split(/\r?\n|\r/g);
            for(var i = 0; i < sp_value.length; i++)
                st.c("value").t(sp_value[i]).up();
        } else if (this.nodeName.toLowerCase() == "input" && this.type == "checkbox") {
            if (this.checked) {
                st.c("value").t("1");
            } else {
                st.c("value").t("0");
            }
        } else {
            // if this has value then
            st.c("value").t($(this).val()).up();
        }
        st.up();
    });
    st.up();
    return st.tree();
}

function displayResult(elem, result) {
    var status = $(result).find("command").attr("status");
    var kinds = {'prev': 'Prev', 'next': 'Next', 'complete': 'Complete'};

    $(elem).empty();
    $(result).find("command > *").each(function(index, e) {
        if ($(e).is("note")) {
            addNote(elem, $(e).text(), $(e).attr("type"));
        } else if ($(e).is("x[xmlns=jabber:x:data]")) {
            addForm(elem, e);
        }
    });
    if (status == "executing") {
        for (kind in kinds) {
            input = $("<input type='button' disabled='disabled' value='" + kinds[kind] + "'/>").click(function() {
                if (kind == 'prev')
                    executeCommand(kind, false, function(e) { displayResult(elem, e) });
                else
                    executeCommand(kind, serializeToDataform('form'), function(e) { displayResult(elem, e) });
            });
            if ($(result).find('actions ' + kind).length > 0)
                input.removeAttr("disabled");
            $(elem).append(input);
        }

        $("<input type='button' id='executeButton' value='Execute'/>").click(function() {
            executeCommand("execute", serializeToDataform('form'), function(e) { displayResult(elem, e) });
        }).appendTo(elem);

        $("<input type='button' value='Cancel'/>").click(function() {
            cancelCommand(function(e) { displayResult(elem, e) });
        }).appendTo(elem);
    } else {
        input = $("<input type='button' value='Start over'/>").bind("click", function() {
            $(elem).empty();
            adhoc_status.sessionid = null;
            adhoc_status.cmdNode = null;
            getCommandNodes(elem);
        });
        $(elem).append(input);
    }
}

function runCommand(item, callback) {
    adhoc_status.cmdNode = $(item).attr("id"); // Save node of executed command (in global var)
    executeCommand("execute", false, function(result) {
        adhoc_status.sessionid = $(result).find("command").attr("sessionid");
        callback(result);
    });
}

function executeCommand(type, childs, callback) {
    if (adhoc_status.sessionid)
        var execIQ = $iq({ type: "set", to: adhoc_status.queryJID, id: connection.getUniqueId() })
            .c("command", { xmlns: Strophe.NS.ADHOC, node: adhoc_status.cmdNode, sessionid: adhoc_status.sessionid, action: type });
    else
        var execIQ = $iq({ type: "set", to: adhoc_status.queryJID, id: connection.getUniqueId() })
            .c("command", { xmlns: Strophe.NS.ADHOC, node: adhoc_status.cmdNode, action: type });
    if (childs)
        execIQ.cnode(childs);
        connection.sendIQ(execIQ, callback);
}

function cancelCommand(callback) {
    executeCommand("cancel", false, callback);
    adhoc_status.cmdNode = null
    adhoc_status.sessionid = null;
}

function getCommandNodes(elem) {
    var nodesIQ = $iq({ type: "get", to: adhoc_status.queryJID, id: connection.getUniqueId() }).c("query", {xmlns: Strophe.NS.DISCO_ITEMS, node: Strophe.NS.ADHOC});
    connection.sendIQ(nodesIQ, function(result) {
        var items = $("<ul></ul>");
        $(elem).append(items);
        $(result).find("item").each(function(index, e) {
            $("<li></li>").append($("<a href='#' id='" + $(e).attr("node") + "'>" + $(e).attr("name") + "</a>").click(function (event) {
                runCommand(this, function (result) { displayResult(elem, result); });
                event.preventDefault();
            })).appendTo(items);
        });
    });
}

function checkFeatures(elem) {
    if (adhoc_status.sessionid)
        cancelCommand();
    featureIQ = $iq({ type: "get", to: adhoc_status.queryJID, id: connection.getUniqueId() }).c("query", {xmlns: Strophe.NS.DISCO_INFO});
    $(elem).empty();
    connection.sendIQ(featureIQ,
        function(result) { /* Callback */
            if ($(result).find("feature[var='" + Strophe.NS.ADHOC + "']").length > 0) {
                getCommandNodes(elem);
            } else {
                $(elem).append("<p>" + adhoc_status.queryJID + " does NOT support AdHoc commands</p>");
            }
        },
        function(result) { /* Errback */
            $(elem).append("<p>Couldn't get list of supported features</p>");
        }
    );
}

function showConnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);        

    button.value = 'connect';
    $('#query').hide();
    pass.show();
    jid.show();
    $('#cred label').show();
    $('#cred br').show();
    $('#output').empty();
}

function showDisconnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);        

    button.value = 'disconnect';
    pass.hide();
    jid.hide();
    $('#cred label').hide();
    $('#cred br').hide();
}

$(document).ready(function () {
    connection = new Strophe.Connection(BOSH_SERVICE);
    if (show_log) {
        $('#log_container').show();
        connection.rawInput = rawInput;
        connection.rawOutput = rawOutput;
    }

    $("#log_toggle").click(function () {
        $("#log").toggle();
      });

    $('#cred').bind('submit', function (event) {
        var button = $('#connect').get(0);
        var jid = $('#jid');
        var pass = $('#pass');        
        localJID = jid.get(0).value;

        if (button.value == 'connect') {
            showDisconnect();
            $('#log').empty();
            connection.connect(localJID,
               pass.get(0).value,
               onConnect);
        } else {
            connection.disconnect();
        }
        event.preventDefault();
    });

    $('#queryForm').bind('submit', function (event) {
        adhoc_status.queryJID = $('#queryJID').val();
        checkFeatures("#output");
        event.preventDefault();
    });
});

window.onunload = window.onbeforeunload = function() {
    if (connection) {
        connection.disconnect();
    }
}
